@@ -816,8 +816,6 @@ contact_list_view_popup_menu_idle_cb (gpointer user_data)
 	}
 
 	if (menu) {
-		g_signal_connect (menu, "deactivate",
-				  G_CALLBACK (gtk_menu_detach), NULL);
 		gtk_menu_attach_to_widget (GTK_MENU (menu),
 					   GTK_WIDGET (data->view), NULL);
 		gtk_widget_show (menu);
@@ -856,7 +854,7 @@ contact_list_view_key_press_event_cb (EmpathyContactListView *view,
 				      GdkEventKey	     *event,
 				      gpointer		      user_data)
 {
-	if (event->keyval == GDK_Menu) {
+	if (event->keyval == GDK_KEY_Menu) {
 		MenuPopupData *data;
 
 		data = g_slice_new (MenuPopupData);
@@ -936,8 +934,6 @@ contact_list_view_call_activated_cb (
 	gtk_menu_shell_append (shell, item);
 	gtk_widget_show (item);
 
-	g_signal_connect (menu, "deactivate",
-			  G_CALLBACK (gtk_menu_detach), NULL);
 	gtk_menu_attach_to_widget (GTK_MENU (menu),
 				   GTK_WIDGET (view), NULL);
 	gtk_widget_show (menu);
@@ -955,29 +951,28 @@ contact_list_view_cell_set_background (EmpathyContactListView *view,
 				       gboolean                is_group,
 				       gboolean                is_active)
 {
-	GdkColor  color;
-	GtkStyle *style;
-
-	style = gtk_widget_get_style (GTK_WIDGET (view));
-
 	if (!is_group && is_active) {
-		color = style->bg[GTK_STATE_SELECTED];
+		GdkRGBA color;
+		GtkStyleContext *style;
+
+		style = gtk_widget_get_style_context (GTK_WIDGET (view));
+
+		gtk_style_context_get_background_color (style, GTK_STATE_FLAG_SELECTED,
+        &color);
 
 		/* Here we take the current theme colour and add it to
 		 * the colour for white and average the two. This
 		 * gives a colour which is inline with the theme but
 		 * slightly whiter.
 		 */
-		color.red = (color.red + (style->white).red) / 2;
-		color.green = (color.green + (style->white).green) / 2;
-		color.blue = (color.blue + (style->white).blue) / 2;
+		empathy_make_color_whiter (&color);
 
 		g_object_set (cell,
-			      "cell-background-gdk", &color,
+			      "cell-background-rgba", &color,
 			      NULL);
 	} else {
 		g_object_set (cell,
-			      "cell-background-gdk", NULL,
+			      "cell-background-rgba", NULL,
 			      NULL);
 	}
 }
@@ -1282,7 +1277,7 @@ contact_list_view_search_key_navigation_cb (GtkWidget *search,
 	GdkEventKey *eventkey = ((GdkEventKey *) event);
 	gboolean ret = FALSE;
 
-	if (eventkey->keyval == GDK_Up || eventkey->keyval == GDK_Down) {
+	if (eventkey->keyval == GDK_KEY_Up || eventkey->keyval == GDK_KEY_Down) {
 		GdkEvent *new_event;
 
 		new_event = gdk_event_copy (event);
