@@ -39,7 +39,6 @@
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-connection-managers.h>
 #include <libempathy/empathy-connectivity.h>
-#include <libempathy/empathy-gsettings.h>
 
 #include <libempathy-gtk/empathy-ui-utils.h>
 #include <libempathy-gtk/empathy-protocol-chooser.h>
@@ -48,7 +47,6 @@
 #include <libempathy-gtk/empathy-account-widget-sip.h>
 #include <libempathy-gtk/empathy-cell-renderer-activatable.h>
 #include <libempathy-gtk/empathy-images.h>
-#include <libempathy-gtk/mx-gtk-light-switch.h>
 
 #include "empathy-accounts-dialog.h"
 #include "empathy-import-dialog.h"
@@ -87,7 +85,6 @@ typedef struct {
   GtkWidget *label_status;
   GtkWidget *image_status;
   GtkWidget *throbber;
-  GtkWidget *enabled_switch;
   GtkWidget *frame_no_protocol;
 
   GtkWidget *treeview;
@@ -108,18 +105,12 @@ typedef struct {
   GtkWidget *spinner;
   gboolean loading;
 
-  /* We have to keep a weak reference on the actual EmpathyAccountWidget, not
-   * just its GtkWidget. It is the only reliable source we can query to know if
+  /* We have to keep a reference on the actual EmpathyAccountWidget, not just
+   * his GtkWidget. It is the only reliable source we can query to know if
    * there are any unsaved changes to the currently selected account. We can't
    * look at the account settings because it does not contain everything that
    * can be changed using the EmpathyAccountWidget. For instance, it does not
-   * contain the state of the "Enabled" checkbox.
-   *
-   * Even if we create it ourself, we just get a weak ref and not a strong one
-   * as EmpathyAccountWidget unrefs itself when the GtkWidget is destroyed.
-   * That's kinda ugly; cf bgo #640417.
-   *
-   * */
+   * contain the state of the "Enabled" checkbox. */
   EmpathyAccountWidget *setting_widget_object;
 
   gboolean  connecting_show;
@@ -218,49 +209,6 @@ accounts_dialog_status_infobar_set_message (EmpathyAccountsDialog *dialog,
   g_free (message_markup);
 }
 
-static void
-accounts_dialog_enable_account_cb (GObject *account,
-    GAsyncResult *result,
-    gpointer user_data)
-{
-  GError *error = NULL;
-
-  tp_account_set_enabled_finish (TP_ACCOUNT (account), result, &error);
-
-  if (error != NULL)
-    {
-      DEBUG ("Could not enable the account: %s", error->message);
-      g_error_free (error);
-    }
-  else
-    {
-      TpAccountManager *am = tp_account_manager_dup ();
-
-      empathy_connect_new_account (TP_ACCOUNT (account), am);
-      g_object_unref (am);
-    }
-}
-
-static void
-accounts_dialog_enable_switch_flipped (MxGtkLightSwitch *sw,
-    gboolean state,
-    EmpathyAccountsDialog *dialog)
-{
-  EmpathyAccountSettings *settings;
-  TpAccount *account;
-
-  settings = accounts_dialog_model_get_selected_settings (dialog);
-  if (settings == NULL)
-    return;
-
-  account = empathy_account_settings_get_account (settings);
-  if (account == NULL)
-    return;
-
-  tp_account_set_enabled_async (account, state,
-      accounts_dialog_enable_account_cb, NULL);
-}
-
 static void
 accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
     TpAccount *account)
@@ -323,19 +271,6 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
   gtk_image_set_from_icon_name (GTK_IMAGE (priv->image_status),
       empathy_icon_name_for_presence (presence), GTK_ICON_SIZE_SMALL_TOOLBAR);
 
-  /* update the enabled switch */
-  g_signal_handlers_block_by_func (priv->enabled_switch,
-      accounts_dialog_enable_switch_flipped, dialog);
-  mx_gtk_light_switch_set_active (MX_GTK_LIGHT_SWITCH (priv->enabled_switch),
-      account_enabled);
-  g_signal_handlers_unblock_by_func (priv->enabled_switch,
-      accounts_dialog_enable_switch_flipped, dialog);
-
-  /* inform the account widget */
-  g_object_set (priv->setting_widget_object,
-      "enabled", account_enabled,
-      NULL);
-
   if (account_enabled)
     {
       switch (status)
@@ -549,16 +484,9 @@ account_dialog_create_settings_widget (EmpathyAccountsDialog *dialog,
   const gchar               *icon_name;
   TpAccount                 *account;
 
-  if (priv->setting_widget_object != NULL)
-    g_object_remove_weak_pointer (G_OBJECT (priv->setting_widget_object),
-        (gpointer *) &priv->setting_widget_object);
-
   priv->setting_widget_object =
       empathy_account_widget_new_for_protocol (settings, FALSE);
 
-  g_object_add_weak_pointer (G_OBJECT (priv->setting_widget_object),
-      (gpointer *) &priv->setting_widget_object);
-
   if (accounts_dialog_has_valid_accounts (dialog))
     empathy_account_widget_set_other_accounts_exist (
         priv->setting_widget_object, TRUE);
@@ -655,79 +583,18 @@ accounts_dialog_setup_ui_to_add_account (EmpathyAccountsDialog *dialog)
 {
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   EmpathyAccountSettings *settings;
-  gchar *str;
-  const gchar *name, *display_name;
-  TpConnectionManager *cm;
-  TpConnectionManagerProtocol *proto;
-  gboolean is_gtalk, is_facebook;
 
-  cm = empathy_protocol_chooser_dup_selected (
-      EMPATHY_PROTOCOL_CHOOSER (priv->combobox_protocol), &proto, &is_gtalk,
-      &is_facebook);
-  if (cm == NULL)
+  settings = empathy_protocol_chooser_create_account_settings (
+      EMPATHY_PROTOCOL_CHOOSER (priv->combobox_protocol));
+  if (settings == NULL)
     return;
 
-  if (is_gtalk)
-    name = "gtalk";
-  else if (is_facebook)
-    name ="facebook";
-  else
-    name = proto->name;
-
-  display_name = empathy_protocol_name_to_display_name (name);
-  if (display_name == NULL)
-    display_name = proto->name;
-
-  /* Create account */
-  /* To translator: %s is the name of the protocol, such as "Google Talk" or
-   * "Yahoo!"
-   */
-  str = g_strdup_printf (_("New %s account"), display_name);
-  settings = empathy_account_settings_new (cm->name, proto->name, str);
-
-  g_free (str);
-
-  if (is_gtalk)
-    {
-      gchar *fallback_servers[] = {
-          "talkx.l.google.com",
-          "talkx.l.google.com:443,oldssl",
-          "talkx.l.google.com:80",
-          NULL};
-
-      gchar *extra_certificate_identities[] = {
-          "talk.google.com",
-          NULL};
-
-      empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
-          NULL, NULL);
-
-      empathy_account_settings_set_strv (settings, "fallback-servers",
-          fallback_servers);
-
-      if (empathy_account_settings_have_tp_param (settings,
-              "extra-certificate-identities"))
-        {
-          empathy_account_settings_set_strv (settings,
-                  "extra-certificate-identities", extra_certificate_identities);
-        }
-    }
-  else if (is_facebook)
-    {
-      empathy_account_settings_set_icon_name_async (settings, "im-facebook",
-          NULL, NULL);
-
-      empathy_account_settings_set_string (settings, "server",
-          "chat.facebook.com");
-    }
-
   accounts_dialog_add (dialog, settings);
   accounts_dialog_model_set_selected (dialog, settings);
 
   gtk_widget_show_all (priv->hbox_protocol);
 
   g_object_unref (settings);
-  g_object_unref (cm);
 }
 
 static void
@@ -1107,22 +974,8 @@ accounts_dialog_row_changed_foreach (GtkTreeModel *model,
     GtkTreeIter *iter,
     gpointer user_data)
 {
-  TpAccount *account;
+  gtk_tree_model_row_changed (model, path, iter);
 
-  gtk_tree_model_get (model, iter, COL_ACCOUNT, &account, -1);
-
-  if (account == NULL)
-    return FALSE;
-
-  if (tp_account_get_connection_status (account, NULL) ==
-      TP_CONNECTION_STATUS_CONNECTING)
-    {
-      /* Only update the row where we have a connecting account as that's the
-       * ones having a blinking icon. */
-      gtk_tree_model_row_changed (model, path, iter);
-    }
-
-  g_object_unref (account);
   return FALSE;
 }
 
@@ -1620,28 +1473,6 @@ accounts_dialog_get_account_iter (EmpathyAccountsDialog *dialog,
   return FALSE;
 }
 
-static void
-select_and_scroll_to_iter (EmpathyAccountsDialog *dialog,
-    GtkTreeIter *iter)
-{
-  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
-  GtkTreeSelection *selection;
-  GtkTreePath *path;
-  GtkTreeModel *model;
-
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
-
-  gtk_tree_selection_select_iter (selection, iter);
-
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
-  path = gtk_tree_model_get_path (model, iter);
-
-  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (priv->treeview), path, NULL,
-      TRUE, 0, 0.5);
-
-  gtk_tree_path_free (path);
-}
-
 static void
 accounts_dialog_model_set_selected (EmpathyAccountsDialog *dialog,
     EmpathyAccountSettings *settings)
@@ -1652,7 +1483,7 @@ accounts_dialog_model_set_selected (EmpathyAccountsDialog *dialog,
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
   if (accounts_dialog_get_settings_iter (dialog, settings, &iter))
-    select_and_scroll_to_iter (dialog, &iter);
+    gtk_tree_selection_select_iter (selection, &iter);
 }
 static void
 accounts_dialog_add (EmpathyAccountsDialog *dialog,
@@ -1799,7 +1630,6 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
   const gchar        *name;
   gboolean            enabled;
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
-  gboolean selected = FALSE;
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
   status = tp_account_get_connection_status (account, NULL);
@@ -1809,16 +1639,7 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
   settings = empathy_account_settings_new_for_account (account);
 
   if (!accounts_dialog_get_account_iter (dialog, account, &iter))
-    {
-      gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-    }
-  else
-    {
-      GtkTreeSelection *selection;
-
-      selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
-      selected = gtk_tree_selection_iter_is_selected (selection, &iter);
-    }
+    gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 
   gtk_list_store_set (GTK_LIST_STORE (model), &iter,
       COL_NAME, name,
@@ -1827,14 +1648,6 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
       COL_ACCOUNT_SETTINGS, settings,
       -1);
 
-  if (selected)
-    {
-      /* We just modified the selected account. Its display name may have been
-       * changed and so it's place in the treeview. Scroll to it so it stays
-       * visible. */
-      select_and_scroll_to_iter (dialog, &iter);
-    }
-
   accounts_dialog_connection_changed_cb (account,
       0,
       status,
@@ -2008,7 +1821,7 @@ accounts_dialog_set_selected_account (EmpathyAccountsDialog *dialog,
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
   if (accounts_dialog_get_account_iter (dialog, account, &iter))
-    select_and_scroll_to_iter (dialog, &iter);
+    gtk_tree_selection_select_iter (selection, &iter);
 }
 
 static void
@@ -2193,7 +2006,7 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
 
   content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
-  gtk_container_add (GTK_CONTAINER (content_area), top_hbox);
+  gtk_box_pack_start (GTK_BOX (content_area), top_hbox, TRUE, TRUE, 0);
 
   g_object_unref (gui);
 
@@ -2288,16 +2101,6 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
   gtk_box_pack_start (GTK_BOX (hbox), priv->image_status, FALSE, FALSE, 3);
   gtk_box_pack_start (GTK_BOX (hbox), priv->label_status, TRUE, TRUE, 0);
 
-  /* enabled switch */
-  align = gtk_alignment_new (0.5, 0.5, 1., 0.);
-  gtk_box_pack_start (GTK_BOX (content_area), align, FALSE, TRUE, 0);
-
-  priv->enabled_switch = mx_gtk_light_switch_new ();
-  gtk_container_add (GTK_CONTAINER (align), priv->enabled_switch);
-  g_signal_connect (priv->enabled_switch, "switch-flipped",
-      G_CALLBACK (accounts_dialog_enable_switch_flipped), dialog);
-  gtk_widget_show_all (align);
-
   /* Tweak the dialog */
   gtk_window_set_title (GTK_WINDOW (dialog), _("Messaging and VoIP Accounts"));
   gtk_window_set_role (GTK_WINDOW (dialog), "accounts");
@@ -2402,7 +2205,6 @@ do_constructed (GObject *object)
   EmpathyAccountsDialog *dialog = EMPATHY_ACCOUNTS_DIALOG (object);
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   GtkTreeModel *model;
-  GSettings *gsettings = g_settings_new (EMPATHY_PREFS_SCHEMA);
 
   accounts_dialog_build_ui (dialog);
   accounts_dialog_model_setup (dialog);
@@ -2419,24 +2221,7 @@ do_constructed (GObject *object)
   tp_account_manager_prepare_async (priv->account_manager, NULL,
       accounts_dialog_manager_ready_cb, dialog);
 
-  if (empathy_import_accounts_to_import ())
-    {
-      gtk_widget_show (priv->button_import);
-
-      if (!g_settings_get_boolean (gsettings, EMPATHY_PREFS_IMPORT_ASKED))
-        {
-          GtkWidget *import_dialog;
-
-          g_settings_set_boolean (gsettings, EMPATHY_PREFS_IMPORT_ASKED, TRUE);
-          import_dialog = empathy_import_dialog_new (GTK_WINDOW (dialog),
-              FALSE);
-          gtk_widget_show (import_dialog);
-        }
-    }
-
   priv->connectivity = empathy_connectivity_dup_singleton ();
-
-  g_object_unref (gsettings);
 }
 
 static void
@@ -2529,11 +2314,8 @@ empathy_accounts_dialog_show_application (GdkScreen *screen,
 
   if (selected_account != NULL)
     {
-      const gchar *account_path;
-
-      account_path = tp_proxy_get_object_path (TP_PROXY (selected_account));
       account_option = g_strdup_printf ("--select-account=%s",
-          &account_path[strlen (TP_ACCOUNT_OBJECT_PATH_BASE)]);
+          tp_account_get_path_suffix (selected_account));
 
       argv[i++] = account_option;
     }
