@@ -60,7 +60,7 @@
 #include <libempathy-gtk/empathy-new-call-dialog.h>
 #include <libempathy-gtk/empathy-log-window.h>
 #include <libempathy-gtk/empathy-presence-chooser.h>
-#include <libempathy-gtk/empathy-sound.h>
+#include <libempathy-gtk/empathy-sound-manager.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 
 #include "empathy-accounts-dialog.h"
@@ -101,11 +101,13 @@ G_DEFINE_TYPE (EmpathyMainWindow, empathy_main_window, GTK_TYPE_WINDOW);
 #define GET_PRIV(self) ((EmpathyMainWindowPriv *)((EmpathyMainWindow *) self)->priv)
 
 struct _EmpathyMainWindowPriv {
+	EmpathyContactList      *contact_manager;
 	EmpathyIndividualStore  *individual_store;
 	EmpathyIndividualView   *individual_view;
 	TpAccountManager        *account_manager;
 	EmpathyChatroomManager  *chatroom_manager;
 	EmpathyEventManager     *event_manager;
+	EmpathySoundManager     *sound_mgr;;
 	guint                    flash_timeout_id;
 	gboolean                 flash_on;
 	gboolean                 empty;
@@ -120,6 +122,7 @@ struct _EmpathyMainWindowPriv {
 	GtkWidget              *presence_toolbar;
 	GtkWidget              *presence_chooser;
 	GtkWidget              *errors_vbox;
+	GtkWidget              *auth_vbox;
 	GtkWidget              *search_bar;
 	GtkWidget              *notebook;
 	GtkWidget              *no_entry_label;
@@ -140,8 +143,13 @@ struct _EmpathyMainWindowPriv {
 	GtkWidget              *edit_context_separator;
 
 	guint                   size_timeout_id;
+
+	/* reffed TpAccount* => visible GtkInfoBar* */
 	GHashTable             *errors;
 
+	/* EmpathyEvent* => visible GtkInfoBar* */
+	GHashTable             *auths;
+
 	/* stores a mapping from TpAccount to Handler ID to prevent
 	 * to listen more than once to the status-changed signal */
 	GHashTable             *status_changed_handlers;
@@ -283,6 +291,138 @@ main_window_flash_start (EmpathyMainWindow *window)
 	main_window_flash_cb (window);
 }
 
+static void
+main_window_remove_auth (EmpathyMainWindow *window,
+			 EmpathyEvent      *event)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+	GtkWidget *error_widget;
+
+	error_widget = g_hash_table_lookup (priv->auths, event);
+	if (error_widget != NULL) {
+		gtk_widget_destroy (error_widget);
+		g_hash_table_remove (priv->auths, event);
+	}
+}
+
+static void
+main_window_auth_add_clicked_cb (GtkButton         *button,
+				 EmpathyMainWindow *window)
+{
+	EmpathyEvent *event;
+
+	event = g_object_get_data (G_OBJECT (button), "event");
+
+	empathy_event_approve (event);
+
+	main_window_remove_auth (window, event);
+}
+
+static void
+main_window_auth_close_clicked_cb (GtkButton         *button,
+				   EmpathyMainWindow *window)
+{
+	EmpathyEvent *event;
+
+	event = g_object_get_data (G_OBJECT (button), "event");
+
+	empathy_event_decline (event);
+	main_window_remove_auth (window, event);
+}
+
+static void
+main_window_auth_display (EmpathyMainWindow *window,
+			  EmpathyEvent      *event)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+	TpAccount *account = event->account;
+	GtkWidget *info_bar;
+	GtkWidget *content_area;
+	GtkWidget *image;
+	GtkWidget *label;
+	GtkWidget *add_button;
+	GtkWidget *close_button;
+	GtkWidget *action_area;
+	GtkWidget *action_table;
+	const gchar *icon_name;
+	gchar *str;
+
+	if (g_hash_table_lookup (priv->auths, event) != NULL) {
+		return;
+	}
+
+	info_bar = gtk_info_bar_new ();
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_QUESTION);
+
+	gtk_widget_set_no_show_all (info_bar, TRUE);
+	gtk_box_pack_start (GTK_BOX (priv->auth_vbox), info_bar, FALSE, TRUE, 0);
+	gtk_widget_show (info_bar);
+
+	icon_name = tp_account_get_icon_name (account);
+	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_SMALL_TOOLBAR);
+	gtk_widget_show (image);
+
+	str = g_markup_printf_escaped ("<b>%s</b>\n%s",
+				       tp_account_get_display_name (account),
+				       _("Password required"));
+
+	label = gtk_label_new (str);
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_widget_show (label);
+
+	g_free (str);
+
+	content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar));
+	gtk_box_pack_start (GTK_BOX (content_area), image, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (content_area), label, TRUE, TRUE, 0);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
+	add_button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (add_button), image);
+	gtk_widget_set_tooltip_text (add_button, _("Provide Password"));
+	gtk_widget_show (add_button);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON);
+	close_button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (close_button), image);
+	gtk_widget_set_tooltip_text (close_button, _("Disconnect"));
+	gtk_widget_show (close_button);
+
+	action_table = gtk_table_new (1, 2, FALSE);
+	gtk_table_set_col_spacings (GTK_TABLE (action_table), 6);
+	gtk_widget_show (action_table);
+
+	action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
+	gtk_box_pack_start (GTK_BOX (action_area), action_table, FALSE, FALSE, 0);
+
+	gtk_table_attach (GTK_TABLE (action_table), add_button, 0, 1, 0, 1,
+			  (GtkAttachOptions) (GTK_SHRINK),
+			  (GtkAttachOptions) (GTK_SHRINK), 0, 0);
+	gtk_table_attach (GTK_TABLE (action_table), close_button, 1, 2, 0, 1,
+			  (GtkAttachOptions) (GTK_SHRINK),
+			  (GtkAttachOptions) (GTK_SHRINK), 0, 0);
+
+	g_object_set_data_full (G_OBJECT (info_bar),
+				"event", event, NULL);
+	g_object_set_data_full (G_OBJECT (add_button),
+				"event", event, NULL);
+	g_object_set_data_full (G_OBJECT (close_button),
+				"event", event, NULL);
+
+	g_signal_connect (add_button, "clicked",
+			  G_CALLBACK (main_window_auth_add_clicked_cb),
+			  window);
+	g_signal_connect (close_button, "clicked",
+			  G_CALLBACK (main_window_auth_close_clicked_cb),
+			  window);
+
+	gtk_widget_show (priv->auth_vbox);
+
+	g_hash_table_insert (priv->auths, event, info_bar);
+}
+
 static void
 main_window_event_added_cb (EmpathyEventManager *manager,
 			    EmpathyEvent        *event,
@@ -290,6 +430,8 @@ main_window_event_added_cb (EmpathyEventManager *manager,
 {
 	if (event->contact) {
 		main_window_flash_start (window);
+	} else if (event->type == EMPATHY_EVENT_TYPE_AUTH) {
+		main_window_auth_display (window, event);
 	}
 }
 
@@ -301,6 +443,11 @@ main_window_event_removed_cb (EmpathyEventManager *manager,
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
 	FlashForeachData data;
 
+	if (event->type == EMPATHY_EVENT_TYPE_AUTH) {
+		main_window_remove_auth (window, event);
+		return;
+	}
+
 	if (!event->contact) {
 		return;
 	}
@@ -385,6 +532,9 @@ main_window_row_deleted_cb (GtkTreeModel      *model,
 			gtk_label_set_markup (GTK_LABEL (priv->no_entry_label), tmp);
 			g_free (tmp);
 
+			gtk_label_set_ellipsize (GTK_LABEL (priv->no_entry_label),
+				PANGO_ELLIPSIZE_END);
+
 			gtk_notebook_set_current_page (
 					GTK_NOTEBOOK (priv->notebook), PAGE_NO_MATCH);
 		}
@@ -588,6 +738,7 @@ main_window_error_display (EmpathyMainWindow *window,
 			  G_CALLBACK (main_window_error_retry_clicked_cb),
 			  window);
 
+	gtk_widget_set_tooltip_text (priv->errors_vbox, error_message);
 	gtk_widget_show (priv->errors_vbox);
 
 	g_hash_table_insert (priv->errors, g_object_ref (account), info_bar);
@@ -598,7 +749,7 @@ main_window_update_status (EmpathyMainWindow *window)
 {
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
 	gboolean connected, connecting;
-	GList *l;
+	GList *l, *children;
 
 	connected = empathy_account_manager_get_accounts_connected (&connecting);
 
@@ -615,6 +766,15 @@ main_window_update_status (EmpathyMainWindow *window)
 	for (l = priv->actions_connected; l; l = l->next) {
 		gtk_action_set_sensitive (l->data, connected);
 	}
+
+	/* Update favourite rooms sensitivity */
+	children = gtk_container_get_children (GTK_CONTAINER (priv->room_menu));
+	for (l = children; l != NULL; l = l->next) {
+		if (g_object_get_data (G_OBJECT (l->data), "is_favorite") != NULL) {
+			gtk_widget_set_sensitive (GTK_WIDGET (l->data), connected);
+		}
+	}
+	g_list_free (children);
 }
 
 static void
@@ -626,6 +786,8 @@ main_window_connection_changed_cb (TpAccount  *account,
 				   GHashTable *details,
 				   EmpathyMainWindow *window)
 {
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+
 	main_window_update_status (window);
 
 	if (current == TP_CONNECTION_STATUS_DISCONNECTED &&
@@ -634,12 +796,12 @@ main_window_connection_changed_cb (TpAccount  *account,
 	}
 
 	if (current == TP_CONNECTION_STATUS_DISCONNECTED) {
-		empathy_sound_play (GTK_WIDGET (window),
+		empathy_sound_manager_play (priv->sound_mgr, GTK_WIDGET (window),
 				    EMPATHY_SOUND_ACCOUNT_DISCONNECTED);
 	}
 
 	if (current == TP_CONNECTION_STATUS_CONNECTED) {
-		empathy_sound_play (GTK_WIDGET (window),
+		empathy_sound_manager_play (priv->sound_mgr, GTK_WIDGET (window),
 				    EMPATHY_SOUND_ACCOUNT_CONNECTED);
 
 		/* Account connected without error, remove error message if any */
@@ -692,7 +854,10 @@ empathy_main_window_finalize (GObject *window)
 
 	g_object_unref (priv->account_manager);
 	g_object_unref (priv->individual_store);
+	g_object_unref (priv->contact_manager);
+	g_object_unref (priv->sound_mgr);
 	g_hash_table_destroy (priv->errors);
+	g_hash_table_destroy (priv->auths);
 
 	/* disconnect all handlers of status-changed signal */
 	g_hash_table_iter_init (&iter, priv->status_changed_handlers);
@@ -950,6 +1115,15 @@ main_window_notify_contact_list_size_cb (GSettings         *gsettings,
 	gtk_radio_action_set_current_value (priv->normal_with_avatars, value);
 }
 
+static void
+main_window_edit_search_contacts_cb (GtkCheckMenuItem  *item,
+				     EmpathyMainWindow *window)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+
+	empathy_individual_view_start_search (priv->individual_view);
+}
+
 static void
 main_window_view_show_map_cb (GtkCheckMenuItem  *item,
 			      EmpathyMainWindow *window)
@@ -1098,6 +1272,8 @@ main_window_favorite_chatroom_menu_add (EmpathyMainWindow *window,
 
 	name = empathy_chatroom_get_name (chatroom);
 	menu_item = gtk_menu_item_new_with_label (name);
+	g_object_set_data (G_OBJECT (menu_item), "is_favorite",
+			GUINT_TO_POINTER (TRUE));
 
 	g_object_set_data (G_OBJECT (chatroom), "menu_item", menu_item);
 	g_signal_connect (menu_item, "activate",
@@ -1422,7 +1598,8 @@ main_window_connection_items_setup (EmpathyMainWindow *window,
 	GObject       *action;
 	guint          i;
 	const gchar *actions_connected[] = {
-		"room",
+		"room_join_new",
+		"room_join_favorites",
 		"chat_new_message",
 		"chat_new_call",
 		"chat_add_contact",
@@ -1533,7 +1710,6 @@ static void
 empathy_main_window_init (EmpathyMainWindow *window)
 {
 	EmpathyMainWindowPriv    *priv;
-	EmpathyContactList       *list_iface;
 	EmpathyIndividualManager *individual_manager;
 	GtkBuilder               *gui;
 	GtkWidget                *sw;
@@ -1551,6 +1727,8 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	priv->gsettings_ui = g_settings_new (EMPATHY_PREFS_UI_SCHEMA);
 	priv->gsettings_contacts = g_settings_new (EMPATHY_PREFS_CONTACTS_SCHEMA);
 
+	priv->sound_mgr = empathy_sound_manager_dup_singleton ();
+
 	gtk_window_set_title (GTK_WINDOW (window), _("Contact List"));
 	gtk_window_set_role (GTK_WINDOW (window), "contact_list");
 	gtk_window_set_default_size (GTK_WINDOW (window), 225, 325);
@@ -1560,6 +1738,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	gui = empathy_builder_get_file (filename,
 				       "main_vbox", &priv->main_vbox,
 				       "errors_vbox", &priv->errors_vbox,
+				       "auth_vbox", &priv->auth_vbox,
 				       "ui_manager", &priv->ui_manager,
 				       "view_show_offline", &show_offline_widget,
 				       "view_show_protocols", &priv->show_protocols,
@@ -1603,6 +1782,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 			      "edit_accounts", "activate", main_window_edit_accounts_cb,
 			      "edit_personal_information", "activate", main_window_edit_personal_information_cb,
 			      "edit_preferences", "activate", main_window_edit_preferences_cb,
+			      "edit_search_contacts", "activate", main_window_edit_search_contacts_cb,
 			      "help_about", "activate", main_window_help_about_cb,
 			      "help_debug", "activate", main_window_help_debug_cb,
 			      "help_contents", "activate", main_window_help_contents_cb,
@@ -1628,6 +1808,8 @@ empathy_main_window_init (EmpathyMainWindow *window)
 					      g_object_unref,
 					      NULL);
 
+	priv->auths = g_hash_table_new (NULL, NULL);
+
 	priv->status_changed_handlers = g_hash_table_new_full (g_direct_hash,
 							       g_direct_equal,
 							       NULL,
@@ -1673,7 +1855,12 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	gtk_container_add (GTK_CONTAINER (item), priv->throbber);
 	priv->throbber_tool_item = GTK_WIDGET (item);
 
-	list_iface = EMPATHY_CONTACT_LIST (empathy_contact_manager_dup_singleton ());
+	/* XXX: this class is designed to live for the duration of the program,
+	 * so it's got a race condition between its signal handlers and its
+	 * finalization. The class is planned to be removed, so we won't fix
+	 * this before then. */
+	priv->contact_manager = EMPATHY_CONTACT_LIST (
+			empathy_contact_manager_dup_singleton ());
 	individual_manager = empathy_individual_manager_dup_singleton ();
 	priv->individual_store = empathy_individual_store_new (
 			individual_manager);
@@ -1688,11 +1875,9 @@ empathy_main_window_init (EmpathyMainWindow *window)
 			EMPATHY_INDIVIDUAL_FEATURE_ALL);
 
 	priv->butterfly_log_migration_members_changed_id = g_signal_connect (
-			list_iface, "members-changed",
+			priv->contact_manager, "members-changed",
 			G_CALLBACK (main_window_members_changed_cb), window);
 
-	g_object_unref (list_iface);
-
 	gtk_widget_show (GTK_WIDGET (priv->individual_view));
 	gtk_container_add (GTK_CONTAINER (sw),
 			   GTK_WIDGET (priv->individual_view));
